'use strict';

var lezerCql = require('lezer-cql');
var language = require('@codemirror/language');
var autocomplete = require('@codemirror/autocomplete');

const cqlKeywordCompletionSource = () => {
    return autocomplete.ifIn(["BinaryComparisonPredicate", "BooleanExpression"], autocomplete.completeFromList([
        { displayLabel: "and", label: "and ", type: "keyword" },
        { displayLabel: "or", label: "or ", type: "keyword" },
        { displayLabel: "not", label: "not ", type: "keyword" },
        { displayLabel: "like", label: "like ", type: "keyword" },
        { displayLabel: "between", label: "between ", type: "keyword" },
        { displayLabel: "is", label: "is ", type: "keyword" },
        { displayLabel: "in", label: "in ", type: "keyword" },
    ]));
};
const cqlConstantCompletionSource = () => {
    return autocomplete.ifIn(["BinaryComparisonPredicate", "BooleanExpression"], autocomplete.completeFromList([
        { label: "null", type: "constant" },
        { label: "EMPTY", type: "constant" },
        { label: "true", type: "constant" },
        { label: "false", type: "constant" },
    ]));
};
function applyFunctionCompletion(view, completion, from, to) {
    const text = completion.label + "()";
    const pos = from + text.length - 1;
    view.dispatch(autocomplete.insertCompletionText(view.state, text, from, to));
    view.dispatch({
        selection: { anchor: pos, head: pos },
    });
}
function createFunctionCompetion(label) {
    return {
        label,
        type: "function",
        apply: applyFunctionCompletion,
    };
}
const functionCompletionList = [
    "T_AFTER",
    "T_BEFORE",
    "T_CONTAINS",
    "T_DISJOINT",
    "T_DURING",
    "T_EQUALS",
    "T_FINISHEDBY",
    "T_FINISHES",
    "T_INTERSECTS",
    "T_MEETS",
    "T_METBY",
    "T_OVERLAPPEDBY",
    "T_OVERLAPS",
    "T_STARTEDBY",
    "T_STARTS",
    "S_INTERSECTS",
    "S_EQUALS",
    "S_DISJOINT",
    "S_TOUCHES",
    "S_WITHIN",
    "S_OVERLAPS",
    "S_CROSSES",
    "S_CONTAINS",
    "A_EQUALS",
    "A_CONTAINS",
    "A_CONTAINEDBY",
    "A_OVERLAPS",
    "CASEI",
    "ACCENTI",
    "DATE",
    "TIMSTAMP",
    "INTERVAL",
    "POINT",
    "LINESTRING",
    "POLYGON",
    "MULTIPOINT",
    "MULTILINESTRING",
    "MULTIPOLYGON",
    "GEOMETRYCOLLECTION",
    "BBOX",
].map(createFunctionCompetion);
const cqlBuiltinFunctionCompletionSource = () => {
    return autocomplete.ifIn(["Identifier"], autocomplete.completeFromList(functionCompletionList));
};

function cql() {
    const extensions = [
        cqlKeywordCompletionExtension,
        cqlConstantCompletionExtension,
        cqlBuiltinFunctionCompletionExtension,
    ];
    return new language.LanguageSupport(cqlLanguage, extensions);
}
const cqlLanguage = language.LRLanguage.define({
    parser: lezerCql.parser.configure({}),
    languageData: {
        closeBrackets: { brackets: ["(", "'", '"'] },
    },
});
const cqlKeywordCompletionExtension = cqlLanguage.data.of({
    autocomplete: cqlKeywordCompletionSource(),
});
const cqlConstantCompletionExtension = cqlLanguage.data.of({
    autocomplete: cqlConstantCompletionSource(),
});
const cqlBuiltinFunctionCompletionExtension = cqlLanguage.data.of({
    autocomplete: cqlBuiltinFunctionCompletionSource(),
});

exports.cql = cql;
exports.cqlLanguage = cqlLanguage;
